// Copyright (c) 2016, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

library reflectable.test.mock_tests.check_literal_transform_test;

/// Test the literal output of the transformation for a few simple cases.

import 'package:barback/barback.dart';
import "package:reflectable/test_transform.dart";
import "package:reflectable/transformer.dart";
import "package:unittest/unittest.dart";

var useReflect = [
  {
    "a|main.dart": """
import 'package:reflectable/reflectable.dart';
import 'main.reflectable.dart';

class MyReflectable extends Reflectable {
  const MyReflectable(): super(newInstanceCapability);
}

@MyReflectable()
class A {}

main() {
  initializeReflectable();
  InstanceMirror instanceMirror = myReflectable.reflect(new A());
}
"""
  },
  {
    "a|main.reflectable.dart": """
// This file has been generated by the reflectable package.
// https://github.com/dart-lang/reflectable.

import "dart:core";
import 'main.dart' as prefix0;

// ignore:unused_import
import "package:reflectable/mirrors.dart" as m;
// ignore:unused_import
import "package:reflectable/src/reflectable_transformer_based.dart" as r;
// ignore:unused_import
import "package:reflectable/reflectable.dart" show isTransformed;

final _data = {
  const prefix0.MyReflectable(): new r.ReflectorData(
      <m.TypeMirror>[
        new r.NonGenericClassMirrorImpl(
            r"A",
            r".A",
            7,
            0,
            const prefix0.MyReflectable(),
            const <int>[-1],
            null,
            null,
            -1,
            {},
            {},
            {r"": (b) => () => b ? new prefix0.A() : null},
            -1,
            -1,
            const <int>[-1],
            null,
            {})
      ],
      null,
      null,
      <Type>[prefix0.A],
      1,
      {},
      {},
      null,
      [
        const [0, 0, null]
      ])
};

final _memberSymbolMap = null;

initializeReflectable() {
  if (!isTransformed) {
    throw new UnsupportedError(
        "The transformed code is running with the untransformed "
        "reflectable package. Remember to set your package-root to "
        "'build/.../packages'.");
  }
  r.data = _data;
  r.memberSymbolMap = _memberSymbolMap;
}
"""
  }
];

const String package = "a";

checkTransform(List<Map<String, String>> maps) async {
  Map<String, String> inputs = maps[0];
  Map<String, String> expectedOutputs = maps[1];

  for (String inputName in inputs.keys) {
    String inputContents = inputs[inputName];
    TestTransform transform =
        new TestTransform(inputName, inputContents, package);
    ReflectableTransformer transformer =
        new ReflectableTransformer.asPlugin(new BarbackSettings({
      "entry_points": ["main.dart"],
      "formatted": true,
    }, BarbackMode.RELEASE));

    // Test `declareOutputs`.
    TestDeclaringTransform declaringTransform =
        new TestDeclaringTransform(inputName);
    transformer.declareOutputs(declaringTransform);
    expect(declaringTransform.outputs, new Set.from(expectedOutputs.keys));
    expect(declaringTransform.consumed, false);

    testApply(ReflectableTransformer transformer) async {
      await transformer.apply(transform);
      Map<String, String> outputs = await transform.outputMap();
      if (transform.messages.isNotEmpty) {
        // It is very difficult to detect what went wrong if, say, package-bots
        // fail here, so we print the messages so that they will be in the log.
        for (MessageRecord messageRecord in transform.messages) {
          String level = null;
          switch (messageRecord.level) {
            case LogLevel.INFO:
              level = "Info";
              break;
            case LogLevel.FINE:
              level = "Fine";
              break;
            case LogLevel.WARNING:
              level = "Warning";
              break;
            case LogLevel.ERROR:
              level = "Error";
              break;
          }
          assert(level != null);
          print("[$level from check_literal_transform_test]:\n"
              "${messageRecord.message}");
        }
      }
      expect(transform.messages.isEmpty, true);
      expect(outputs.length, expectedOutputs.length);
      outputs.forEach((key, value) {
        // The error message is nicer when the strings are compared separately
        // instead of comparing Maps.
        expect(value, expectedOutputs[key]);
      });
    }

    // Test `apply`.
    await testApply(transformer);

    // Test that transformation tolerates a duplicate entry point.
    ReflectableTransformer duplicateTransformer =
        new ReflectableTransformer.asPlugin(new BarbackSettings({
      "entry_points": ["main.dart", "main.dart"],
      "formatted": true,
    }, BarbackMode.RELEASE));
    await testApply(duplicateTransformer);
  }
}

main() async {
  test("Check transforms", () async {
    await checkTransform(useReflect);
  });
}
